## Distributed as part of the supplementary material for the manuscript
##
## "Mean and median bias reduction: A concise review and application
## to adjacent-categories logit models"
##
## Author: Ioannis Kosmidis
## Date: 05 December 2021
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!

library("brglm2")
library("ordinal")
library("parallel")


## ================
## Example 5
## ================
data("wine", package = "ordinal")
## Combine categories as in Christensen (2019, Section 4.3)
wine2 <- within(wine, {
   rating_comb3 <- rating
   levels(rating_comb3) <- c("1", "2-4", "2-4", "2-4", "5")
})
true_fit <- bracl(rating_comb3 ~ temp + contact, data = wine2, parallel = TRUE, type = "AS_mean")
z_formula <- ~ contact
true_oss <- ordinal_superiority(true_fit, z_formula, measure = "gamma", bc = FALSE)
true_oss

## Simulation
n_cores <- 8
n_simu <- 10000
set.seed(123)
seeds <- sample(1:(n_simu * 10^4), n_simu, replace = FALSE)
## If
length(unique(seeds)) == n_simu
## is TRUE continue

oml <- obr <- as.list(rep(NA, n_simu))
res <- mclapply(1:n_simu, function(r) {
    if (r %% 10 == 0) cat(r, "\n")
    set.seed(seeds[r])
    df <- simulate(true_fit)
    fml <- update(true_fit, weights = .weights, data = df, type = "ML")
    fbr <- update(true_fit, weights = .weights, data = df, type = "AS_mean")
    list(ml = ordinal_superiority(fml, z_formula, measure = "gamma", bc = FALSE),
         br = ordinal_superiority(fbr, z_formula, measure = "gamma", bc = TRUE))
}, mc.cores = n_cores)

## Relative bias
true_g <- true_oss[, "gamma"]
gml <- rowMeans(sapply(res, function(x) x$ml[, "gamma"]))
gbr <- rowMeans(sapply(res, function(x) x$br[, "gamma"]))
100 * (gml - true_g) /  true_g
100 * (gbr - true_g) /  true_g

## PU
rowMeans(sapply(res, function(x) x$ml[, "gamma"] < true_g))
rowMeans(sapply(res, function(x) x$br[, "gamma"] < true_g))

## Coverage probabilities
ciml <- sapply(res, function(x) true_g > x$ml[, c("2.5 %")] & true_g < x$ml[, c("97.5 %")])
cibr <- sapply(res, function(x) true_g > x$br[, c("2.5 %")] & true_g < x$br[, c("97.5 %")])
rowMeans(ciml)
rowMeans(cibr)


