## Distributed as part of the supplementary material for the manuscript
##
## "Mean and median bias reduction: A concise review and application
## to adjacent-categories logit models"
##
## Author: Ioannis Kosmidis
## Date: 05 December 2021
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!

library("brglm2")
library("colorspace")
library("ggplot2")
library("enrichwith")
library("detectseparation")

## =================
## Example 1
## =================
set.seed(123)
n <- 100
x1 <- rnorm(n, 0, 2)
x2 <- runif(n, 0, 2)
y <- as.numeric(x1 + 2 * x2 > 0)

m_ML <- glm(y ~ x1 + x2, family = binomial())
(cML <- coef(m_ML))
update(m_ML, method = "detect_separation")

## =================
## Figure 1
## =================
df <- data.frame(y = as.character(y), x1, x2)
ggplot(df) +
    geom_point(aes(x = x1, y = x2, col = y), alpha = 0.7, size = 2) +
    geom_abline(aes(intercept = - cML[1] / cML[3], slope = - cML[2] / cML[3]),
                lty = 2) +
    theme_minimal() +
    labs(x = expression(x[2]), y = expression(x[3])) +
    theme(legend.position = "top")


## =================
## Example 2
## =================
m_meanRB <- update(m_ML, method = "brglm_fit", type = "AS_mean")
m_medianRB <- update(m_ML, method = "brglm_fit", type = "AS_median")

cmeanRB <- coef(m_meanRB)
cmedianRB <- coef(m_medianRB)

## =================
## Table 1
## =================
coef(summary(m_ML))
coef(summary(m_meanRB))
coef(summary(m_medianRB))



## =================
## Section 6.2
## =================
mt_meanRB <- update(m_meanRB, y ~ x1 + I(x1 + x2))
## The diffrence in mean BR estimates
diff(coef(m_meanRB)[3:2])
## should be equal to the estimate of the difference of mean BR estimates
coef(mt_meanRB)[2]

## The same is not true for median BR
## The median BR estimate of beta_2 - beta_3 is not
diff(coef(m_medianRB)[3:2])
## but rather
mt_medianRB <- update(m_medianRB, y ~ x1 + I(x1 + x2))
coef(mt_medianRB)[2]


## =================
## Section 6.4
## =================
info <- get_information_function(m_ML)
bias <- get_bias_function(m_ML)
scores <- get_score_function(m_ML)

scores_zeta <- function(zeta) {
    scores(log(zeta)) / zeta
}
info_zeta <- function(zeta) {
    diag(1/zeta) %*% info(log(zeta)) %*% diag(1/zeta)
}
bias_zeta <- function(zeta, beta_from_mean_br = TRUE) {
    beta <- log(zeta)
    ((1 - isTRUE(beta_from_mean_br)) * bias(beta) + 0.5 * diag(solve(info(beta)))) * zeta
}

## zeta^*
exp(cmeanRB) - bias_zeta(exp(cmeanRB), TRUE)

## zeta^**
v <- diag(solve(info(cRB <- coef(m_meanRB))))
round(exp(cmeanRB) / (1 + v/2), 3)

