## Distributed as part of the supplementary material for the manuscript
##
## "Mean and median bias reduction: A concise review and application
## to adjacent-categories logit models"
##
## Author: Ioannis Kosmidis
## Date: 05 December 2021
## Licence: GPL 2 or greater
## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!

library("brglm2")
library("ordinal")
library("dplyr")
library("tibble")
library("ggplot2")
library("VGAM")


## ================
## Table 2
## =================

data("wine", package = "ordinal")
wine_table <- ftable(xtabs(~ contact + temp + rating, data = wine))
wine_table


## ================
## Figure 3
## ================
ell <- function(freq, categories, epsilon = 0) {
    n <- length(freq)
    ll <- log((freq[-n] + epsilon) / (freq[-1] + epsilon))
    cats <- paste(categories[-n], "vs",  categories[-1])
    data.frame(local_logits = ll, categories = cats)
}

epsilon <- 0.5
wine_tibble <-
    as_tibble(wine_table) %>%
    mutate(freq = Freq, temperature = temp) %>%
    group_by(temperature, contact) %>%
    arrange(temperature, contact, rating) %>%
    summarize(ell(freq, rating, 0.5))

ggplot(wine_tibble %>% mutate(local_logits = ifelse(is.infinite(local_logits), NA, local_logits)), aes(group = temperature:contact)) +
    geom_line(aes(x = categories, y = local_logits, lty = contact, colour = temperature)) +
    geom_point(aes(x = categories, y = local_logits, colour = temperature)) +
    theme_minimal() +
    theme(legend.position = "top") +
    scale_colour_grey() +
    labs(x = "rating categories", y = "empirical local logits")


## ================
## Table 3
## ================
wine_freqs <- wine_table[, c(1, 2, 3, 4, 5)]
wine_covs <- as.data.frame(wine_table)[c(1, 3, 2, 4), c(1, 2)]
fit_vgam <- vglm(wine_freqs ~ wine_covs$temp + wine_covs$contact,
                     family = acat(reverse = TRUE, parallel = FALSE))
vgam_est <- coef(summary(fit_vgam))[, c(1:2)]
vgam_est <- format(cbind(vgam_est[1:4, ], vgam_est[4 + 1:4, ], vgam_est[8 + 1:4, ]),
                   digits = 1, nsmall = 2, scientific = FALSE)
print(vgam_est)
logLik(fit_vgam)

## Let's make the tolerance a bit stricter
fit_vgam_s <- update(fit_vgam, epsilon = 1e-09)
vgam_s_est <- coef(summary(fit_vgam_s))[, c(1:2)]
vgam_s_est <- format(cbind(vgam_s_est[1:4, ], vgam_s_est[4 + 1:4, ], vgam_s_est[8 + 1:4, ]),
                   digits = 1, nsmall = 2, scientific = FALSE)
print(vgam_s_est)


## ================
## Example 4
## ================
### Mean BR estimation
fit_npo <- bracl(rating ~ temp + contact, data = wine, parallel = FALSE, type = "AS_mean")
fit_npo_est <- coef(summary(fit_npo))[, c(1:2)]
fit_npo_est <- format(cbind(fit_npo_est[1:4, ], fit_npo_est[4 + 1:4, ], fit_npo_est[8 + 1:4, ]),
                   digits = 1, nsmall = 2, scientific = FALSE)
max(abs(fit_npo$grad), na.rm = TRUE)

## Computation of Wald statistic
npar <- length(coef(fit_npo))
ncat <- fit_npo$ncat
ncontr <- ncat - 2
cc <- t(contr.sum(ncat - 1))
contr_matrix <- rbind(cbind(cc, matrix(0, nrow = ncontr, ncol = ncat - 1)),
                      cbind(matrix(0, nrow = ncontr, ncol = ncat - 1), cc))

diffs <- contr_matrix %*% coef(fit_npo)[-(1:(ncat - 1))]
info_diffs <- solve(contr_matrix %*% vcov(fit_npo)[-(1:(ncat - 1)), -(1:(ncat - 1))]  %*% t(contr_matrix))

## Statistic and chisq quantile
(W <- t(diffs) %*% info_diffs %*% diffs)
qchisq(0.95, npar - ncontr * 2)

## ML, mean BR and median BR estimates of beta for the PO version of the ACL model
fit_po_ml <- update(fit_npo, parallel = TRUE, type = "ML")
fit_po_medi <- update(fit_npo, parallel = TRUE, type = "AS_median")
fit_po_mean <- update(fit_npo, parallel = TRUE, type = "AS_mean")

format(coef(summary(fit_po_ml))[-c(1:4), 1:2], digits = 1, nsmall = 2, scientific = FALSE)
format(coef(summary(fit_po_medi))[-c(1:4), 1:2], digits = 1, nsmall = 2, scientific = FALSE)
format(coef(summary(fit_po_mean))[-c(1:4), 1:2], digits = 1, nsmall = 2, scientific = FALSE)



